<?php

namespace InSegment\ApiCore\Models;

use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;

class SliceRemap extends SupplementaryTable
{
    /**
     * Run the migrations.
     *
     * @param array $enabledTables
     * @param bool $inMemory
     * @param string|null $uid
     * @return void
     */
    public static function establish(array $enabledTables, bool $inMemory, $uid = null)
    {
        Schema::create((new static([], $uid))->getTable(), function (Blueprint $table) use ($enabledTables, $inMemory) {
            if ($inMemory) {
                $table->engine = 'MEMORY';
            }
            
            $table->unsignedBigInteger('substitution_uuid');
            $table->unsignedInteger('original_id')->nullable();
            $table->unsignedInteger('destination_id')->nullable();
            $table->unsignedInteger('query_id');
            $table->enum('table', $enabledTables);
            $table->primary(['substitution_uuid'], 'slice_remap_primary');
            $table->index(['table'], 'slice_remap_table_index');
            $table->index(['original_id'], 'slice_remap_original_id_index');
            $table->index(['destination_id'], 'slice_remap_destination_id_index');
            $table->index(['table', 'query_id'], 'slice_remap_lookup_index');
            $table->unique(['table', 'original_id'], 'slice_remap_table_unique');
        });
    }
    
    /**
     * Generate identifiers for tables with auto-increment primary keys and update the record of UUID mapping
     * 
     * @param array $allowedUuidables [
     *     $table => $keyName,
     *     ...
     * ]
     * @param string $generationsTbl
     * @return null
     */
    public static function generateIdentifiers($allowedUuidables, $generationsTbl)
    {
        list($gensDb, $gensTableName) = explode('.', $generationsTbl);
        
        foreach ($allowedUuidables as $table => $keyName) {
            DB::statement("SELECT * FROM `{$table}` WHERE `{$keyName}` > (SELECT MAX(`{$keyName}`) FROM `{$table}`) FOR UPDATE");
            DB::statement("SET @ai = (SELECT `AUTO_INCREMENT` FROM `information_schema`.`tables` WHERE `table_name` = '{$table}' AND `table_schema` = DATABASE())");
            DB::statement("UPDATE `{$gensDb}`.`{$gensTableName}` SET `destination_id` = (@ai:= @ai + 1) - 1 WHERE `table` = '{$table}'");
        }
    }
    
    /**
     * Update statement replacing UUIDs with auto-increment primary keys
     * 
     * @param string $tempTable
     * @param string $generationsTbl
     * @param string $key
     * @return string
     */
    public static function getUUIDUpdateStatement(string $tempTable, string $generationsTbl, string $key)
    {
        list($databaseTemp, $tempTableName) = explode('.', $tempTable);
        list($databaseGens, $gensTableName) = explode('.', $generationsTbl);
        return
            "UPDATE `{$databaseTemp}`.`{$tempTableName}` as `temp_table`"
            . " INNER JOIN `{$databaseGens}`.`{$gensTableName}` as `gens_table` ON `gens_table`.`substitution_uuid` = `temp_table`.`{$key}`"
            . " SET `temp_table`.`{$key}` = `gens_table`.`destination_id`;";
    }
    
}
